
// Cecible_ESP32_utilitaire_v1.22       YLC  09/06/21
// A compiler avec type de carte "DOIT ESP32 DEVKIT V1"
// Utilitaire de calcul de l'angle de calage du lidar (en degrés). 
// Ce programme est a exécuter une seule fois après contruction de la cible et du boitier lidar
// Il sert a calculer précisément l'angle formé par le calage moteur du lidar et le côté de la cible.  
// La mesure obtenue doit être reportée dans la constante "angdep" du prog Cecible_ESP32_v1.22
// Mettre une cale ronde avec son centre exactement sur l'anle droit de la cible vue côté joueurs.
//
//                                     ▼ <--- Lidar
//                                   / | \
//          Côté de référence ---> /   |   \  <---  Côté de référence utilisateur
//          trigonométrique      /     |     \ 
//                              <--- Cible ---☻ <--- cale ronde pour détection
//                               \     |     /
//                                 \   |   /
//                                   \ | /
//                                     V
//
//                                  Joueurs   
//  
// Composition : ESP32 Wroom 32 + Buzzer + Lidar tf mini plus  + moteur pàp 28BYJ-48 + contrôleur ULN2003A
// Hardware    :                + relais reed  + 2x switchs fin de course + détecteur optique à fourche
//             :                + convertisseur de niveau RX/TX <> 3v/5v + Oled 128x64 
// ---------------------------------------------------------------------------------------

// écran Oled 
#include <U8g2lib.h>         // librairie de polices pour écran Oled 
U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0);  

// contrôle du calibrage
const int ledctl = 2;        // builtin led pour contrôle de calage du lidar (allumé si pas calé)

// buzzer
#include "EasyBuzzer.h"       // Attention : utilise le channel ledcpwm 0
const int buzz = 13;          // buzzer connecté sur pin 13 

// fixation de la cible sur le boitier
const int fixcib = 27;         // détecteur de fixation de la cible connecté sur pin 27

// module lidar TF mini plus  sur port série #2 (pins 25 et 27)
#include <TFMPlus.h>          // Include TFMini Plus Library v1.4.1
TFMPlus Lidar;                // Create a TFMini Plus object
int16_t tfDist = 0;    // Distance to object in centimeters
int16_t tfFlux = 0;    // Strength or quality of return signal
int16_t tfTemp = 0;    // Internal temperature of Lidar sensor chip
int dis = 0;            // distance brute mesurée
int dist = 0;           // distance mesurée valide
int dislid = 0;         // cumul des mesures faite par le lidar 

// moteur
#include <TinyStepper_28BYJ_48.h>
const int mot1 = 32;        // assignations des pins de commmande moteur  pin 32
const int mot2 = 33;        // pin 33
const int mot3 = 25;        // pin 25
const int mot4 = 26;        // pin 26  
TinyStepper_28BYJ_48 moteur;       // instance moteur (2048 pas par tour)
#define PI 3.1415926535897932384626433832795
long step90 = 512;      // nb steps pour balayer 90° (2048 pas / tour)
long angobs;            // angle moyen de la position de l'obstacle (en pas) (vers centre de l'obstacle)
long stepfin;           // pas de fin de balayage pour détection de la balle 
long stepcal;           // pas de la position de calibrage par rapport au coté droit de la cible vue des joueurs (négatif)
float angdep;           // angle de calage (en degrés) fourni par cet utilitaire

// détecteurs pour calage moteur
const int lopt = 12;     // capteur optique de calage sur pin 12

// autres variables
long i,j,k,l,m = 0;

//__________________________________________________________________

void sonner(int freq, int duree) {
  EasyBuzzer.singleBeep(freq,duree);      // faire sonner la cible
  delay(duree);
  EasyBuzzer.stopBeep();
}
//__________________________________________________________________

void mesure()  {
  dis = dist;            // pour ne pas prendre en compte la distance si lecture non valide
  dislid = 0;
  m = 0;
  for (l = 0; l < 5; l++) { 
    Lidar.getData(tfDist,tfFlux,tfTemp);          // Récuperation de la distance mesurée en cm         
    if ((tfDist > 30) && (tfDist < 130)) {       // si mesure valide  (sinon mesure inchangée) 
      dislid = dislid + tfDist;      
      m++;  
    }
  }
  if (dislid > 0) {
    dis = dislid / m;                    // distance moyenne de 5 mesures                            
  } 
}
//__________________________________________________________________

void calcdist() {
  if (dis < dist) {              //  Si distance diminue (centrage sur l'obstacle)
   sonner((2000 - (dist * 4)),30);  // detection sonore de l'obstacle
   dist = dis;
   angobs = j;                  // angle de détection la plus courte (en pas)
  }  
  if (dis > dist) {              // dès que la distance remonte => fin de détection
   j = stepfin;                  // pour sortir de la boucle
  }    
}
//__________________________________________________________________

void calibrage() {
  // NB : on tient compte qu'une saleté pourrait occulter le détecteur optique et en fausser sa lecture
  // d'où les tests des fins de course qui ne devraient normalement pas être sollicités.
  while (digitalRead(lopt) == LOW) {  // si le détecteur optique est occulté mais peut être mal calé 
    digitalWrite(ledctl, LOW);        // éteindre la led de controle 
    moteur.moveRelativeInSteps(1);    // touner le lidar en sens horaire de 1 pas
  }
  while (digitalRead(lopt) == HIGH) {  // tant que détecteur optique n'est pas occulté (donc à 5V)
    digitalWrite(ledctl, HIGH);        // allumer la led de controle (la led s'éteindra dès que le lidar est calé)
    moteur.moveRelativeInSteps(-1);    // pour faire balayage anti-horaire de 1 pas
  }
  digitalWrite(ledctl, LOW);         // éteindre la led de controle, la lidar est bien calé   
  moteur.setCurrentPositionInSteps(stepcal);  // attribuer la position de calage au moteur
  moteur.disableMotor();             // arrêt du moteur    
  stepfin = stepcal + step90;        // limiter de fin de balayage à 90°
}
//__________________________________________________________________

void setup() {
  EasyBuzzer.update();         // correction bug dans lib EasyBuzzer
  Serial.begin(115200);
  u8g2.begin(); 
  u8g2.setFont(u8g2_font_courB10_tf);  // choix de la police (CF: https://github.com/olikraus/u8g2/wiki/fntlistall)
  Serial2.begin (115200, SERIAL_8N1, 16, 17) ;   // port série 2: Lidar sur pins 16 et 17
  Lidar.begin(&Serial2);
  Lidar.sendCommand( SET_FRAME_RATE, FRAME_1000); // nombre de mesures du Lidar par seconde 
  moteur.connectToPins(mot1,mot2,mot3,mot4);
  moteur.disableMotor();                 // mettre moteur à l'arrêt au démarrage
  moteur.setSpeedInStepsPerSecond(500);
  moteur.setAccelerationInStepsPerSecondPerSecond(1000);
  pinMode(ledctl,OUTPUT);      // led de la carte (pour contrôle visuel calage du lidar)
  digitalWrite(ledctl,LOW);    // ...éteinte au démarrage
  EasyBuzzer.setPin(13);       // pin 13 : Pin buzzer
  pinMode(lopt,INPUT_PULLUP);  // détecteur optique de positionnement (à 0v au repos)
  pinMode(fixcib,INPUT_PULLUP); // détecteur fixation cible (à 0v si bien fixée)
}
//__________________________________________________________________

void loop() {  
  calibrage();
  dist = 9999;                          // init dist stockée 
  for (j = stepcal; j < stepfin; j++) {   // balayage 
    moteur.moveToPositionInSteps(j);
    mesure();    
    calcdist();                           // cherche la distance minimale de la balle     
  }
  angdep = (360.00 / 2048.00) * (angobs - stepcal);  // conversion angle en nb de pas en degrés
  moteur.moveToPositionInSteps(stepcal);  // retour lidar en position de calage    
  moteur.disableMotor();                  //arrêt du moteur  
  u8g2.clearBuffer();
  u8g2.drawStr(10,10,"UTILITAIRE");     // Afficher le nom du programme 
  u8g2.drawStr(0,22,"CeCible v1.22");  
  u8g2.setCursor(0,40);
  u8g2.print("Angle calage =");   
  u8g2.setCursor(0,57);
  u8g2.print(angdep,2); 
  u8g2.setCursor(53,57);  
  u8g2.print(" degres");     
  u8g2.sendBuffer(); 
  sonner(1200,50);                       
  delay(3000);                         // on répète la mesure toutes les 3 secondes
}
//______________________FIN PROGRAMME____________________________________________
